using GLib;
using Gtk;
using GnomeVFS;

public class Application
{
	private static string config_dir;
	
	private SList window_list;
	
	private int _win_count = 0;
	public int win_count {
		get { return _win_count; }
		set {
			_win_count = value;
			if ( _win_count <= 0 ) {
				Gtk.main_quit ();
			}
		}
	}
	
	public static string get_config_dir () {
		return config_dir;
	}
	
	[NoArrayLength ()]
	public construct (string[] args) {
		int i;
		string startup_id = Environment.get_variable ("DESKTOP_STARTUP_ID");
		
		Gtk.init (out args);
		GnomeVFS.init ();
		var instance = UniqueApp.get_with_startup_id ("scratchpad", startup_id);
	
		if ( instance.is_running () ) {
			bool first = true;
			i = 0;
			foreach (string doc in args) {
				if (first) { first = false; continue; }
				instance.open_uri (doc);
				i++;
			}
			if ( i == 0 ) {
				Gdk.Global.notify_startup_complete ();
				instance.new_document ();
			}
			Gdk.Global.notify_startup_complete ();
		} else {
			instance.message += on_message_received;
			add_documents (args);
		}
		
		if ( win_count > 0 ) {
			Gtk.AboutDialog.set_url_hook (url_hook, null, null);
			Gtk.AboutDialog.set_email_hook (email_hook, null, null);
			Gtk.main ();
		}
		GnomeVFS.shutdown ();
	}
	
	private static void url_hook (AboutDialog dialog, string link, pointer data) {
		GLib.Spawn.command_line_async ("gnome-open " + link, null);
	}
	
	private static void email_hook (AboutDialog dialog, string email, pointer data) {
		GLib.Spawn.command_line_async ("gnome-open mailto:" + email, null);
	}
	
	private void open_project (string startup_id = null) {
		// open a project window when no arguments have been passed to the process
		// this is not implemented yet...
		
		string message = "This would create a new project oriented window, but this feature is not yet implemented. Please use Scratchpad from your file manager or terminal to open a document directly instead.";
		var d = new MessageDialog (null, (DialogFlags)0, MessageType.INFO, ButtonsType.OK, message);
		if ( startup_id != null )
			UniqueApp.set_window_startup_id (d, startup_id);
			
		// this is blocking which kinda sucks, but so does the entire existance of this dialog
		d.run ();
		d.destroy ();
	}
	
	private void open_document (string uri, string startup_id = null) {
		foreach ( BaseWindow window in window_list ) {
			if ( window.is_document (uri) ) {
				window.present_with_time (time ());
				return;
			}
		}
	
		win_count = win_count + 1;
		var w = new DocumentWindow (uri);
		if ( startup_id != null )
			UniqueApp.set_window_startup_id (w, startup_id);
		w.show_all ();
		window_list.append (w);
		w.destroy += w => { window_list.remove (w); win_count = win_count - 1; };
	}
	
	private void add_documents (string[] docs) {
		int i = 0;
		bool first = true;

		foreach (string uri in docs) {
			if (first) { first = false; continue; }
			open_document (uri);
			i++;
		}
		if ( i == 0 )
			open_project ();
	}
	
	static int main (string[] args) {
		config_dir = Environment.get_home_dir () + "/.gnome2/scratchpad";
		
		Dir.create_with_parents ( Application.get_config_dir () + "/metadata/", 0750 );
		var app = new Application (args);	
	}
	
	private void on_message_received (UniqueApp app, UniqueAppCommand command, string data, string startup_id, uint workspace) {
		if ( command == UniqueAppCommand.OPEN ) {
			open_document (data, startup_id);
		} else if ( command == UniqueAppCommand.NEW ) {
			open_project (startup_id);
		}
	}
}
